@@ -12,7 +12,7 @@ from operator import itemgetter
 from functools import partial
 from json import loads
 
-from xapian import Query, QueryParser, UnimplementedError, DocNotFoundError
+from xapian import Query, QueryParser, DocNotFoundError
 
 from . import snowball
 from .exc import ValidationError, PrefixError
@@ -229,7 +229,8 @@ class Database(object):
                  writable=True,
                  overwrite=False,
                  spelling=True,
-                 replicated=False):
+                 replicated=False,
+                 inmem=False):
         self.db_path = path
         self._writable = writable
         self._overwrite = overwrite
@@ -237,7 +238,13 @@ class Database(object):
         self.replicated = replicated
         self.type_map = {}
         self.parsers_by_language = {}
+        self.relevance_prefixes = {}
+        self.boolean_prefixes = {}
+        self.values = {}
+        self.value_sorts = {}
         self.query_cache = LRUDict(limit=self.query_cache_limit)
+        self.inmem = inmem
+        self._value_count = 0
 
         if isinstance(path, basestring):
             if writable:
@@ -259,6 +266,20 @@ class Database(object):
 
         self.reopen()
 
+    def get_query_parser(self, language, default_op, check_cache=True, 
+                         retry_limit=RETRY_LIMIT):
+        qp = None
+        if check_cache and not self.inmem:
+            qp = self.parsers_by_language.get(language)
+            if qp is not None:
+                return qp
+        def prepare_op():
+            return self.prepare_query_parser(language, default_op)
+        qp = self.retry_if_modified(prepare_op, retry_limit)
+        if check_cache and not self.inmem:
+            self.parsers_by_language[language] = qp
+        return qp
+
     def close(self):
         self.backend.close()
 
@@ -278,10 +299,15 @@ class Database(object):
         raise TypeError("No schema defined for %s" % repr(otype))
 
     def _get_value_count(self):
+        if self.inmem:
+            return self._value_count
         return int(self.backend.get_metadata(self.value_count_name))
 
     def _set_value_count(self, count):
-        self.backend.set_metadata(self.value_count_name, str(count))
+        if self.inmem:
+            self._value_count = count
+        else:
+            self.backend.set_metadata(self.value_count_name, str(count))
 
     value_count = property(_get_value_count, _set_value_count)
 
@@ -585,6 +611,7 @@ class Database(object):
                                   "using replication.")
             self.close()
             self.backend = xapian.Database(self.db_path)
+            # reset cached parsers to new database object
             for parser in self.parsers_by_language.itervalues():
                 parser.set_database(self.backend)
 
@@ -621,7 +648,7 @@ class Database(object):
         qp = QueryParser()
         qp.set_database(self.backend)
         qp.set_default_op(default_op)
-
+        
         if self.boolean_prefixes:
             for key, value in self.boolean_prefixes.items():
                 qp.add_boolean_prefix(key, value)
@@ -668,46 +695,44 @@ class Database(object):
         return self._metadata_keyset != self._get_metadata_keyset()
 
     def _get_metadata_keyset(self, retry_limit=RETRY_LIMIT):
-        try:
-            op = lambda: set(self.backend.metadata_keys())
-            # don't recurse into refresh here, just reopen and retry
-            return self.retry_if_modified(op, retry_limit, False)
-        except UnimplementedError:
-            # Inmemory backends don't expose metadata, so just return
-            # since they can't persist anything anyway!
-            return set()
+        if self.inmem:
+            return False
+        op = lambda: set(self.backend.metadata_keys())
+        # don't recurse into refresh here, just reopen and retry
+        return self.retry_if_modified(op, retry_limit, False)
 
     def meta_refresh(self, retry_limit=RETRY_LIMIT):
-        self.parsers_by_language = {}
-        self.relevance_prefixes = {}
-        self.boolean_prefixes = {}
-        self.values = {}
-        self.value_sorts = {}
-        self.query_cache = LRUDict(limit=self.query_cache_limit)
+        if not self.inmem:
+            self.parsers_by_language = {}
+            self.relevance_prefixes = {}
+            self.boolean_prefixes = {}
+            self.values = {}
+            self.value_sorts = {}
+            self.query_cache = LRUDict(limit=self.query_cache_limit)
 
-        self._metadata_keyset = self._get_metadata_keyset()
-        for k in self._metadata_keyset:
-            op = lambda: self.backend.get_metadata(k)
-            val = self.retry_if_modified(op, retry_limit, False)
-            if k.startswith(self.relevance_prefix):
-                prefix = k[len(self.relevance_prefix):]
-                self.relevance_prefixes[prefix] = val
-            elif k.startswith(self.boolean_prefix):
-                prefix = k[len(self.boolean_prefix):]
-                self.boolean_prefixes[prefix] = val
-            elif k.startswith(self.value_prefix):
-                value = k[len(self.value_prefix):]
-                self.values[value] = int(val)
-            elif k.startswith(self.value_sort_prefix):
-                value = k[len(self.value_sort_prefix):]
-                self.value_sorts[value] = val
-        try:
-            # hit the property to refresh this value
-            count = self.value_count
-            count = count  # pyflakes
-        except ValueError:
-            if self._writable:
-                self.value_count = 0
+            self._metadata_keyset = self._get_metadata_keyset()
+            for k in self._metadata_keyset:
+                op = lambda: self.backend.get_metadata(k)
+                val = self.retry_if_modified(op, retry_limit, False)
+                if k.startswith(self.relevance_prefix):
+                    prefix = k[len(self.relevance_prefix):]
+                    self.relevance_prefixes[prefix] = val
+                elif k.startswith(self.boolean_prefix):
+                    prefix = k[len(self.boolean_prefix):]
+                    self.boolean_prefixes[prefix] = val
+                elif k.startswith(self.value_prefix):
+                    value = k[len(self.value_prefix):]
+                    self.values[value] = int(val)
+                elif k.startswith(self.value_sort_prefix):
+                    value = k[len(self.value_sort_prefix):]
+                    self.value_sorts[value] = val
+            try:
+                # hit the property to refresh this value
+                count = self.value_count
+                count = count  # pyflakes
+            except ValueError:
+                if self._writable:
+                    self.value_count = 0
 
     def querify(self, query,
                 language=None,
@@ -738,17 +763,14 @@ class Database(object):
                 query = query.lower()
                 if translit:
                     query = query.encode(translit)
-                qp = self.parsers_by_language.get(language)
-                if not qp:
-                    def prepare_op():
-                        return self.prepare_query_parser(language, default_op)
-                    qp = self.retry_if_modified(prepare_op, retry_limit)
-                    self.parsers_by_language[language] = qp
 
+                qp = self.get_query_parser(language, default_op, 
+                                           retry_limit=retry_limit)
                 def query_op():
                     return qp.parse_query(query, parser_flags)
                 result = self.retry_if_modified(query_op, retry_limit)
-                self.query_cache[cache_key] = result
+                if not self.inmem:
+                    self.query_cache[cache_key] = result
                 return result
         else:
             return reduce(partial(Query, default_op),
@@ -1006,10 +1028,9 @@ class Database(object):
         """
         Describe the parsed query.
         """
+        qp = self.get_query_parser(language, default_op,
+                                   retry_limit=RETRY_LIMIT)
         def op():
-            qp = self.parsers_by_language.get(language)
-            if not qp:
-                qp = self.prepare_query_parser(language, default_op)
             return qp.parse_query(query, default_parser_flags) 
         return str(self.retry_if_modified(op, retry_limit))
 
@@ -1021,10 +1042,9 @@ class Database(object):
         Suggest a query string with corrected spelling.
         """
         self.reopen()
+        qp = self.get_query_parser(language, default_op, 
+                                   retry_limit=RETRY_LIMIT)
         def op():
-            qp = self.parsers_by_language.get(language)
-            if not qp:
-                qp = self.prepare_query_parser(language, default_op)
             qp.parse_query(query, QueryParser.FLAG_SPELLING_CORRECTION)
             return qp.get_corrected_query_string().decode('utf8')
         return self.retry_if_modified(op, retry_limit)
